# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdio.h>
   # include <stdlib.h>

   # include "command_line.h"
   # include "cwrite.h"
   # include "datachain.h"
   # include "dataline.h"
   # include "datasystem.h"
   # include "failure.h"
   # include "header_line.h"
   # include "message.h"
   # include "screen.h"
   # include "string.h"
# endif
# if GLOBAL
   Datasystem * _datasystem_pointer[DATASYSTEM_CARDINAL];
   Datasystem * _datasystem;
   Datasystem * _datasystem_main;

   int datasystem_count = 0;
   int datasystem_ordinal = 0;
   int datasystem_ordinal_main;

   int datasystem_filename_memsize;
# endif
# if PROTO
   void datasystem_set();
# endif

void datasystem_alloc()
   { for ( int i = 0; i < DATASYSTEM_CARDINAL; i++ )
         { _datasystem_pointer[i] = (Datasystem *) calloc(1, sizeof(Datasystem));
         }
     datasystem_filename_memsize = entry_line_memsize;
     return;
   }

void datasystem_shift_left()
   { if ( datasystem_ordinal == DATASYSTEM_SELECTION )
        { return;
        }
     if ( datasystem_ordinal == DATASYSTEM_FIRST )
        { return;
        }
     datasystem_ordinal--;
     datasystem_set();
     header_line_refresh();
     screen_redraw();
     return;
   }
void datasystem_shift_right()
   { if ( datasystem_ordinal == DATASYSTEM_SELECTION )
        { return;
        }
     //if ( datasystem_ordinal == datasystem_count - 1 )
     if ( datasystem_ordinal == datasystem_count )
       { return;
       }
     datasystem_ordinal++;
     datasystem_set();
     header_line_refresh();
     screen_redraw();
     return;
   }

void datasystem_defrag()
   { int i = datasystem_ordinal;
     Datasystem * _datasystem_tmp = _datasystem_pointer[i];
     //for ( i = datasystem_ordinal; i < datasystem_count - 1; i++ )
     for ( i = datasystem_ordinal; i < datasystem_count; i++ )
        { _datasystem_pointer[i] = _datasystem_pointer[i+1];
        }
     _datasystem_pointer[i] = _datasystem_tmp;
     datasystem_set();
     header_line_refresh();
     screen_redraw();
     return;
   }

void datasystem_new()
   { // datasystem_ordinal = datasystem_count++;
     datasystem_ordinal = ++datasystem_count;
     return;
   }
void datasystem_init(int dataline_max_size)
   {    // -- expects datasystem_ordinal to be properly set --
     int k = datasystem_ordinal;
      
     _datasystem =  _datasystem_pointer[k];
     _memset(_datasystem, 0, sizeof(Datasystem));

     _datasystem->dataline_max_size = dataline_max_size;

     _datasystem->_datachain = (Datachain *) _datachain_creat(_datasystem);
     _datasystem->_datachain_first = _datasystem->_datachain;
     _datasystem->_datachain_last = _datasystem->_datachain;
     _datasystem->_filename = (char *) calloc(1, datasystem_filename_memsize);
     _datasystem->ordinal = k;
     _datasystem->dataline_offset = 0;
     _datasystem->cursor_y = screen_top;
     _datasystem->cursor_x = screen_left;
     _datasystem->status_register = 0;
     _datasystem->status_register &= ~TYPE_MODE;
     _datasystem->status_register |= INS_MODE;

     return;
   }

void datasystem_set()
   { int k = datasystem_ordinal;

     _datasystem = _datasystem_pointer[k];

     __datachain_first = & _datasystem->_datachain_first;
     __datachain = & _datasystem->_datachain;
     __datachain_last = & _datasystem->_datachain_last;
     _dataline_offset = & _datasystem->dataline_offset;
     _cursor_y = & _datasystem->cursor_y;
     _cursor_x = & _datasystem->cursor_x;
     _status_register = & _datasystem->status_register;
     __datachain_detach_start = & _datasystem->_datachain_detach_start;
     __datachain_detach_end = & _datasystem->_datachain_detach_end;

     return;
   }

void datasystem_verify(Datasystem * _datasystem)
   { int sid = 100;

     Datachain * _datachain = _datasystem->_datachain_first;
     Datachain * _datachain_next;

     if ( ! _datachain )
        { fail(sid, _message[52]);
        }
     //while ( _datachain->_next )
     while ( _datachain != _datasystem->_datachain_last )
        { _datachain_next = _datachain->_next;
           if ( _datachain_next->_prev != _datachain )
              { fail(sid, _message[53]);
              }
           if ( _datachain_next->ordinal != _datachain->ordinal + 1 )
              { fail(sid, _message[54]);
              }
           if ( _datachain->size < 0 )
              { fail(sid, _message[55]);
              }
          _datachain = _datachain_next;  
         }
     if ( _datachain != _datasystem->_datachain_last )
        { fail(sid, _message[56]);
        }
     return;
   }
void datasystem_print()
   { Datachain * _datachain = _datasystem->_datachain_first;

     printf("_datachain_first = %x\n", _datasystem->_datachain_first);
     printf("_datachain = %x\n", _datasystem->_datachain);
     printf("_datachain_last = %x\n", _datasystem->_datachain_last);
     while ( _datachain )
        {  datachain_print(_datachain);
          _datachain = _datachain->_next;
        }

     return;     
   }

void datasystem_realloc()
   { int dataline_max_size = _datasystem->dataline_max_size;
     Datachain * _datachain = * __datachain_first;

     while ( _datachain )
        {    // -- alloc and clean one symbol more --
          _datachain->_line = (char *) realloc(_datachain->_line, dataline_max_size + dataline_extend_size+1);
          _memset(_datachain->_line + dataline_max_size, ' ', dataline_extend_size+1);
          _datachain = _datachain->_next;
        }
     _datasystem->dataline_max_size += dataline_extend_size;

     return;
   }
