# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdio.h>

   # include "block.h"
   # include "bookmark.h"
   # include "color_theme.h"
   # include "command.h"
   # include "command_line.h"
   # include "cursor.h"
   # include "cwrite.h"
   # include "datachain.h"
   # include "dataline.h"
   # include "datasystem.h"
   # include "key.h"
   # include "mycurses.h"
   # include "message.h"
   # include "preferences.h"
   # include "screen.h"
   # include "scroll.h"
   # include "search.h"
   # include "selection.h"
   # include "status_line.h"
   # include "view.h"
# endif
# if GLOBAL
   int * _status_register;
# endif
# if PROTO
   void key_ctrl(int c);
   void key_move(int c);
   void key_alt();
   void key_func(int c);
# endif
# if TEST
   //# define TEST_GET_INPUT
# endif

void get_input()
   { int c = 0;
     
     do
        { if ( ! status_line_busy )
             { status_line_refresh();
             }
          c = curses_getch();
          if ( status_line_busy )          
             { status_line_refresh();
               status_line_busy = 0;
             }
          # ifdef TEST_GET_INPUT
             status_line_vprintf("%d", c);
          # endif
          if ( c < 27 )
             { key_ctrl(c);
               continue;
             }
         if ( c == 27 )
             { key_alt();
               continue;
             }
         if ( c > 256 )
             { key_move(c);
               continue;
             }
         if ( ! (* _status_register & TYPE_MODE) )
            { continue;
            }
         dataline_fill(c, 1);
          //dataline_fill(c);
          //if ( * _cursor_x < screen_right )
          //   { (* _cursor_x)++;
          //   }
          //screen_redraw();
        }
     while ( ! exit_request && ! return_request );
     return;
   }

void key_ctrl(int c)
   { switch (c)
        { case 0:
             command_line_enter();
             break;
          case CTRL_A:
             command_block_attach();
             break;
          case CTRL_B:
             //datachain_echo(* __datachain);
             colors_browse();
             break;
          case CTRL_C:
             command_block_collapse();
             break;
          case CTRL_D:
             command_block_detach();
             break;
          case CTRL_E:
             command_block_expand();
             break;
          case CTRL_F:
             search_mode = SEARCH_FIND;
             search_submode = SEARCH_START;
             command_search();
             break;
          case CTRL_G:
             dataline_goto();
             break;
          case CTRL_H:
              cursor_backspace();
              break;
          case CTRL_I:
              dataline_fill(' ', _preferences->tab_value);
              break;
          case CTRL_J:
              command_insert_line();
              break;
          case CTRL_K:
              dataline_remove();
              break;
          case CTRL_L:
              search_mode = SEARCH_LOCATE;
              search_submode = SEARCH_START;
              command_search();
              break;
          case CTRL_M:
              command_insert_line();
              break;
          case CTRL_N:
              //command_new();
              view_expand();
              break;
          case CTRL_O:
              command_open(0);
              break;
          case CTRL_P:
              view_collapse();
              break;
          case CTRL_Q:
              command_exit();
              break;
          case CTRL_R:
              search_submode = SEARCH_START;
              command_replace();
              break;
          case CTRL_S:
              command_save_direct();
              break;
          case CTRL_T:
              search_submode = SEARCH_CONTINUE;
              command_search();
              break;
          case CTRL_U:
              dataline_delete_to_eol();
              break;
          case CTRL_V:
              //show_date();
              selection_paste();
              break;
          case CTRL_W:
              command_close();
              break;
          case CTRL_X:
              show_coords();
              break;
          case CTRL_Y:
              show_date();
              break;
          case 26:
          case 27:
          case 28:
          case 29:
          case 30:
          case 31:
          case 32:
              colors_browse();
              break;
          default:
              break;
        }
      return;
    }
void key_move(int c)
   { switch (c)
        { case KEY_LEFT:
             if ( * _status_register & SCROLL_STATE )
                { scroll_left();
                }
             else
                { cursor_left();
                }
             break;
          case KEY_RIGHT:
             if ( * _status_register & SCROLL_STATE )
                { scroll_right();
                }
             else
                { cursor_right();
                }
             break;
          case KEY_UP:
             if ( * _status_register & SCROLL_STATE )
                { scroll_up();
                }
             else
                { cursor_up();
                }
             break;
          case KEY_DOWN:
             if ( * _status_register & SCROLL_STATE )
                { scroll_down();
                }
             else
                { cursor_down();
                }
             break;
          case KEY_HOME:
             if ( * _status_register & SCROLL_STATE )
                { scroll_home();
                }
             else
                { cursor_home();
                }
             break;
          case KEY_END:
             if ( * _status_register & SCROLL_STATE )
                { scroll_end();
                }
             else
                { cursor_end();
                }
             break;
          case KEY_NPAGE:
             if ( * _status_register & SCROLL_STATE )
                { scroll_npage();
                }
             else
                { cursor_npage();
                }
             break;
          case KEY_PPAGE:
             if ( * _status_register & SCROLL_STATE )
                { scroll_ppage();
                }
             else
                { cursor_ppage();
                }
             break;
          case KEY_DC:
             cursor_delete();
             break;
          case KEY_IC:
             if ( * _status_register & TYPE_MODE )
                { * _status_register ^= INS_MODE;
                }
             break;
          case KEY_BACKSPACE:
             cursor_backspace();
             break;
          default:
             key_func(c);
             break;
        }
     return;
   }
void key_alt()
   { int c = curses_getch();
     //status_line_print("yes");

     if ( c >= '1' && c <= '9' )
        { bookmark_goto(c-49);
          return;
        }
     switch (c)
        { case 'h':
             cursor_left();
             break;
          case 'b':
             bookmark_add();
             break;
          case 'c':
             bookmark_clear();
             break;
        }
     return;
   }
void key_func(int c)
   { Datachain * _datachain_block_begin, * _datachain_block_end;
     Datachain * _datachain;
     int block_level;

     switch (c)
        { case KEY_F(1):
             selection_mark();
             break;
          case KEY_F(2):
             if ( datasystem_ordinal == DATASYSTEM_SELECTION )
                { selection_view_out();
                }
             else
                { selection_view_in();
                }
             break;
          case KEY_F(3):
             block_level = block_locate(* __datachain, & _datachain_block_begin, & _datachain_block_end);
             switch (block_level)
                { case BLOCK_NO_SYNTAX:
                     status_line_print(_message[40]);
                     return;
                  case BLOCK_ERROR:
                     status_line_print(_message[8]);
                     return;
                  case BLOCK_NONE:
                     status_line_print(_message[9]);
                     return;
                }
             _datachain = * __datachain;
             if ( ! _datachain_block_begin->invisible )
                { block_collapse_max(_datachain);
                }
             else
                { block_expand_max();
                }
             break;
          case KEY_F(4):
             block_level = block_locate(* __datachain, & _datachain_block_begin, & _datachain_block_end);
             switch (block_level)
                { case BLOCK_NO_SYNTAX:
                     status_line_print(_message[40]);
                     return;
                  case BLOCK_ERROR:
                     status_line_print(_message[8]);
                     return;
                  case BLOCK_NONE:
                     status_line_print(_message[9]);
                     return;
                }
             _datachain = * __datachain;
             if ( ! _datachain_block_begin->invisible )
                { block_collapse(_datachain_block_begin, _datachain_block_end);
                }
             else
                { block_expand(_datachain_block_begin, _datachain_block_end);
                }
             break;
          case KEY_F(5):
             * _status_register ^= SCROLL_STATE;
             break;
          case KEY_F(9):
             if ( _datasystem->read_only )
                { break;
                }
             * _status_register ^= TYPE_MODE;
             break;
          case KEY_F(10):
             if ( * _status_register & TYPE_MODE )
                { * _status_register ^= INS_MODE;
                }
             break;
          case KEY_F(11):
             datasystem_shift_left();
             break;
          case KEY_F(12):
             datasystem_shift_right();
             break;
        }
     return;
   }
