# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdlib.h>
   # include <stdarg.h>
   # include <time.h>

   # include "color_theme.h"
   # include "cursor.h"
   # include "cwrite.h"
   # include "datasystem.h"
   # include "message.h"
   # include "mycurses.h"
   # include "screen.h"
   # include "status_line.h"
   # include "string.h"
# endif
# if CONST
   # define STATUS_MESSAGE 26
# endif
# if GLOBAL
   char * _status_line;
   char * _status_line_message;

   char * _type_str = " Type";
   char * _view_str = " View      ";
   char * _ins_str = " Ins ";
   char * _ovr_str = " Ovr ";
   char * _scroll_str = " Scroll";

   bool status_line_busy = 0;
   int status_line_message_space;
# endif
# if PROTO
   void status_line_form();
   void status_line_redraw();
   void status_line_clear();
   void status_line_time();
# endif

void status_line_init()
   { _status_line = (char *) malloc(screen_cols);
     _memset(_status_line, ' ', screen_cols);
     _status_line_message = _status_line + STATUS_MESSAGE;
     status_line_message_space = screen_cols - STATUS_MESSAGE - 7;
     return;
   }
void status_line_help()
   { status_line_clear();
     line_copy(_status_line, _message[49]);
     status_line_time();
     status_line_redraw();
     return;
   }

void status_line_clear()
   { _memset(_status_line, ' ', screen_cols);
     return;
   }
void status_line_form()
   { if ( ! datasystem_count )
        { return;
        }
     char * _status_line_point = _status_line;

     if ( * _status_register & TYPE_MODE )
        { line_copy(_status_line_point, _type_str);
          _status_line_point += 5;
          if ( * _status_register & INS_MODE )
             { line_copy(_status_line_point, _ins_str);
             }
          else
             { line_copy(_status_line_point, _ovr_str);
             }
          _status_line_point += 5;
        }
     else
        { line_copy(_status_line_point, _view_str);
          _status_line_point += 10;
        }
     if ( * _status_register & MODIFY_STATE )
        { * _status_line_point = '*';
        }
     else
        { * _status_line_point = ' ';
        }
     _status_line_point += 5;
     if ( * _status_register & DETACH_STATE )
        { * _status_line_point++ = '#';
        }
     else
        { * _status_line_point++ = ' ';
        }
     if ( * _status_register & SCROLL_STATE )
        { line_copy(_status_line_point, _scroll_str);
        }
     return;
   }
void status_line_time()
   { time_t current_time = time(0);
     struct tm * _local_time = localtime(& current_time);
     char * _status_line_point = _status_line + screen_cols - 6;
     strftime(_status_line_point, 6, "%I:%M", _local_time);
     return;
   }

void status_line_overflow()
   { status_line_print(_message[44]);
     return;
   }

void status_line_print(char * _message)
   { status_line_clear();

     status_line_form();
     status_line_time();
     int message_len = string_len(_message);
     if ( message_len > status_line_message_space )
        { return status_line_overflow();
        }
     line_copy(_status_line_message,  _message);
     status_line_redraw();
     status_line_busy = 1;
     return;
   }
void status_line_print_raw(char * _message)
   { status_line_clear();
     line_copy(_status_line_message, _message);
     status_line_busy = 1;
     status_line_redraw();

     return;
   }

void status_line_vprintf(char * _template, ...)
   { status_line_clear();

     va_list ap;
     va_start(ap, _template);
     vsprintf(_status_line_message, _template, ap);

     char * _status_line_tmp = _status_line_message;
     for (int i = 0; i < status_line_message_space; i++)
        { if ( ! * _status_line_tmp )
             { * _status_line_tmp++ = ' ';
             }
          //* _status_line_tmp++ = '/';
        }
     status_line_form();
     status_line_time();

     status_line_busy = 1;
     status_line_redraw();
     return;
   }

void status_line_redraw()
   { curses_mvaddnstr(STATUS_LINE, 0, _status_line, screen_cols);
     curses_mvchgat(STATUS_LINE, 0, screen_cols, A_NORMAL, status_color_pair, 0);
     curses_touchline(stdscr, STATUS_LINE, 1);
     if ( datasystem_count )
        { cursor_move(* _cursor_y, * _cursor_x);
        }
     return;
   }
void status_line_refresh()
   { status_line_clear();
     if ( datasystem_count )
        { status_line_form();
        }
     status_line_time();
     status_line_redraw();

     return;
   }
